<?php

/* DESCRIPTION:
 * This is a PHP library that handles calling CryptoPhoto.
 *    - Main Page
 *        http://cryptophoto.com/
 *    - About Cryptophoto
*         http://cryptophoto.com/about
 *    - Register to CryptoPhoto
 *        http://cryptophoto.com/demo/register/
 *
 * VERSION: 1.120725
 * COPYRIGHT(c) 2011 CryptoPhoto -- http://cryptophoto.com/
 * AUTHORS:
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */


//server URLs
define("CRYPTO_SERVER", "cryptophoto.com/api/");
define("SERVER_ERROR", "error-cryptophoto-not-reachable");
define("REQUEST_ERROR", "error-bad-request");
define("PORT", 80);
define("PORTSSL", 443);

class CryptoPhotoUtils {

  //userdata
  var $privatekey;
  var $publickey;
  var $uid;

  //response properties
  var $is_valid;
  var $sid;
  var $has_token;
  var $error;

  //constructor
  function CryptoPhotoUtils($privatekey, $publickey, $uid) {
    $this->privatekey = $privatekey;
    $this->publickey = $publickey;
    $this->uid = $uid;
  }

  /**
   * encodes the given data into a query string format
   * @param $data - array of string elements to be encoded
   * @return string - encoded request
   */
  function string_encode ($data) {
    $req = "";
    foreach ($data as $key => $value)
            $req .= $key . '=' . urlencode(stripslashes($value)) . '&';

    //cut the last '&'
    $req = substr($req, 0, strlen($req) - 1);
    return $req;
  }


  /**
  * sends an http post-request to an URL
  * @param string $url
  * @param string $udata
  * @return array response*/
  function post_request($url, $udata) {

    if(function_exists('curl_init')) {
      $req = $this->string_encode($udata);
      $cookies = "";
      $response = "";

      $data = array (
                  'http' => array (   
                    'method' => 'POST',
                    'header'=> "Content-type: application/x-www-form-urlencoded\r\n"
                    . "Content-Length: " . strlen($req) . "\r\n"
                    . "Cookie: " . $cookies."\r\n",
                    'content' => $req
                )
            );

      $ch = curl_init ($url);
      curl_setopt ($ch, CURLOPT_POST, true);
      curl_setopt ($ch, CURLOPT_POSTFIELDS, $req);
      curl_setopt ($ch, CURLOPT_RETURNTRANSFER, true);
      $response = curl_exec($ch);
    }

    else {
      $req = $this->string_encode ($udata);
      $url = parse_url($url);
      $extra = "";
      $port  = PORT;

      if ($url['scheme'] == 'https') {
        $extra = "ssl://";
        $port = PORTSSL;
      }

      //extract the components from the URL
      $host = $extra . $url['host'];
      $path = $url['path'];

      //compose the http request
      $http_request  = "POST $path HTTP/1.0\r\n";
      $http_request .= "Host: $host\r\n";
      $http_request .= "Content-Type: application/x-www-form-urlencoded;\r\n";
      $http_request .= "Content-Length: " . strlen($req) . "\r\n";
      $http_request .= "User-Agent: CryptoPhoto/PHP\r\n";
      $http_request .= "\r\n";
      $http_request .= $req;

      //check the response
      $response = '';

      if(false == ($fs = @fsockopen($host, $port, $errno, $errstr, 60))) {
        error_log("Couldn't connect to $host: $errno, $errstr",0);
        return null;
      }

      //write the request on socket
      fwrite($fs, $http_request);

      while (!feof($fs)) {
        //one tcp/ip packet
        $response .= fgets($fs, 1160);
      }

      //close socket
      fclose($fs);
      $response = explode("\r\n\r\n", $response, 2);
    }

    return $response;
}


  /**
  * request a new session
  * @return array */
  function start_session() {

    $time = time();

    $data['publickey'] = $this->publickey;
    $data['uid'] = $this->uid; 
    $data['time'] = $time;
    $data['signature'] = $this->make_signature($this->uid, $this->publickey, $this->privatekey, $time);
  
    if(function_exists('curl_init')) {
      $response = $this->post_request("https://".CRYPTO_SERVER."get/session", $data);
    }
    else {
      $response = $this->post_request("http://".CRYPTO_SERVER."get/session", $data);
    }

    if($response != null && $response != '') {
      //successful request
      
      if(function_exists('curl_init')) {
        $answer = explode("\n", $response);
      }
      else {
        $answer = explode("\n", $response[1]);
      }

      if($answer[0] == 'success') {
        $this->is_valid = TRUE;
        $this->sid = $answer[1];
        $this->has_token = $answer[2];
        return array($this->is_valid, $this->sid, $this->has_token);
      }
      else {
        $this->is_valid = FALSE;
        $this->error = $answer[1];
        return array($this->is_valid, $this->error);
      }
    }

    //server unreachable or bad request
    else {
      $this->is_valid = FALSE;
      $this->error = SERVER_ERROR;
      return array($this->is_valid, $this->error);
    }
  }


  /**
  * generate javascript code for getting a token
  * @return string */
  function get_gen_widget() {
    if($this->sid != null && $this->sid != '') {
      return '<script type="text/javascript" src="https://'.CRYPTO_SERVER.'token?sd='.$this->sid.'"></script>';
    } else {
      die ("Generate session first");
    }
  }

  /**
  * generate javascript code for getting a challenge
  * @return string */
  function get_auth_widget() {
    if($this->sid != null && $this->sid != '') {
      return '<script type="text/javascript" src="https://'.CRYPTO_SERVER.'challenge?sd='.$this->sid.'"></script>';
    } else {
      die ("Generate session first");
    }
  }


  /**
  * post and check a user's response for a given challenge
  * @param string $response_row
  * @param string $response_col
  * @param string $ip
  * @return array */
  function verify_response($selector, $response_row, $response_col, $cp_phc, $ip) {

    if($selector == null || $selector == '') {
      $this->is_valid = FALSE;
      $this->error = SERVER_ERROR;
      //invalid request
      return array("is_valid" => $this->is_valid, "error" => $this->error);
    }
    else {
      if(($response_row == null || $response_row == '' || $response_col == null || $response_col == '') && ($cp_phc == null || $cp_phc == '')) {
        $this->is_valid = FALSE;
        $this->error = SERVER_ERROR;
        //invalid request
        return array("is_valid" => $this->is_valid, "error" => $this->error);
      }
    }

    $time = time();    
    //compose userdata
    $data['publickey'] = $this->publickey;
    $data['time'] = $time;
    $data['signature'] = $this->make_signature($this->uid, $this->publickey, $this->privatekey, $time);
    $data['uid'] = $this->uid;
    $data['response_row'] = $response_row;
    $data['response_col'] = $response_col;
    $data['selector'] = $selector;
    $data['cph']      = $cp_phc;
    $data['ip'] = $ip;

    //post userdata and his challenge response
    if(function_exists('curl_init')) {
      $resp = $this->post_request("https://".CRYPTO_SERVER."verify", $data);
    }
    else {
      $resp = $this->post_request("http://".CRYPTO_SERVER."verify", $data);
    }

    if($resp != null && $resp !='') {

      if(function_exists('curl_init')) {
        $answer = explode("\n", $resp);
      }
      else {
        $answer = explode("\n", $resp[1]);
      }

      //successful request
      if($answer[0] == 'success') {
        //challenge matching successful
        $this->is_valid = TRUE;
        $this->message = $answer[1];
        return array("is_valid" => $this->is_valid, "message" => $this->message);
      }
      else {
        //challenge matching error
        $this->is_valid = FALSE;
        $this->error = $answer[1];
        return array("is_valid" => $this->is_valid, "error" => $this->error = $answer[1]);
      }
    }
    else {
      $this->is_valid = FALSE;
      $this->error = SERVER_ERROR;
      //server unreachable or bad request
      return array("is_valid" => $this->is_valid, "error" => $this->error);
    }
  }

  /**
  * creates the user signature 
  * @param string $uid
  * @param string $publickey
  * @param string $privatekey
  * @param string $time
  * @return string */
  function make_signature($uid, $publickey, $privatekey, $time) {
    return hash_hmac("sha1", $privatekey . $time . $uid . $publickey, $privatekey, false);
  }

}
